#include <Arduino.h>
#include <WiFi.h>

#include "Config.h"
#include "WiFiMeter.h"

namespace Meter {

static bool	enabled = false;

#if defined(APA102_ENABLED)

#include <APA102.h>

constexpr uint8_t	APA102_DATA = 25;
constexpr uint8_t	APA102_CLOCK = 15;
constexpr uint8_t	APA102_COUNT = 6;
constexpr uint8_t	APA102_BRIGHTNESS = 5;


APA102<APA102_DATA, APA102_CLOCK>	ledBar;


static void
setLED(uint8_t brightness)
{
	brightness = brightness & 0x1f;

	ledBar.sendColor(0, 0, 255, brightness);
}


static void
clearBar()
{
	ledBar.startFrame();
	for (int i = 0; i < APA102_COUNT; i++) {
		setLED(0);
	}
	ledBar.endFrame(APA102_COUNT);
}


static void
setLedCount(uint8_t on)
{
	ledBar.startFrame();

	for (int i = 0; i < APA102_COUNT; i++) {
		if (i >= on) {
			setLED(0);
		} else {
			setLED(APA102_BRIGHTNESS);
		}
	}

	ledBar.endFrame(APA102_CLOCK);
}


void
Begin()
{
	enabled = Config::EnableMeter();	
}


void
Enable()
{
	enabled = true;
	Config::SetEnableMeter(true);
}


void
Disable()
{
	enabled = false;
	Config::SetEnableMeter(false);
}


void
Update()
{
	if (!enabled) {
		return;
	}

	int8_t	rssi;
	uint8_t	count = 0;

	if (WiFi.status() != WL_CONNECTED) {
		rssi = -127;
	} else {
		rssi = WiFi.RSSI();
	}


	if (rssi >= -30) {
		count = 6;
	} else if (rssi >= -55) {
		count = 5;
	} else if (rssi >= -67) {
		count = 4;
	} else if (rssi >= -70) {
		count = 3;
	} else if (rssi >= -80) {
		count = 2;
	} else if (rssi >= -90) {
		count = 1;
	} else {
		count = 0;
	}

	setLedCount(count);
}



#else

void	Begin()		{}
void	Enable()	{}
void	Disable()	{}
void	Update()	{}


#endif

}
